/*
 * @author  Giuseppe Stanghellini @ ismar dot cnr dot it
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

import javax.swing.*;
import java.awt.Color;
import org.geo.misc.*;

public class ACSE extends javax.swing.JFrame {
   
    DataSet ds;
    Genetic g;
    ParameterSetup ps;
    FittingFunction ff;
    XYGraphPanel xygp;
    
    /** Creates new form ACSE */
    public ACSE() {
        initComponents();
        xygp = new XYGraphPanel(100, 2);
        grd_display.add(xygp, java.awt.BorderLayout.CENTER);
    }

    public void go()
    {
        ds = new DataSet();
        ds.initializeDataset(dataset_jTextArea);
        xygp.setLayerSymbolType(0, XYGraphPanel.LayerGraph.PointSymbolType.BOX);
        xygp.setLayerColor(0, Color.CYAN);
        xygp.setLayerSymbolType(1, XYGraphPanel.LayerGraph.PointSymbolType.LINE);
        xygp.setLayerColor(1, Color.RED);
        xygp.setTitle("Optimizing function and data");
        
        ps = new ParameterSetup(Integer.valueOf(number_of_parameters.getText()).intValue(), 
                                Integer.valueOf(number_of_bits_per_parameter.getText()).intValue());
        ps.initializeParameters(parameter_values_and_ranges);
        g = new Genetic(Integer.valueOf(number_of_citizens.getText()).intValue(),
                        Integer.valueOf(number_of_selected_citizens.getText()).intValue(),
                        Integer.valueOf(number_of_bits_for_muthagen_control.getText()).intValue(),
                        Integer.valueOf(number_of_max_iterations_not_useful.getText()).intValue(),                        
                        ps);
        
        ff = new FittingFunction();
        ff.function = fitting_function.getText();
        g.setRunningData(ds, ff, Double.valueOf(maximum_acceptable_error_jtextfield.getText()).doubleValue(), xygp, best_parameters_value_jtextarea, error_so_far_jtextarea, iterations_so_far_jtextarea);
        g.optimize();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanel1 = new javax.swing.JPanel();
        jPanel7 = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        grd_display = new javax.swing.JPanel();
        jPanel8 = new javax.swing.JPanel();
        jPanel9 = new javax.swing.JPanel();
        best_parameters_value_jtextarea = new javax.swing.JTextArea();
        error_so_far_jtextarea = new javax.swing.JTextArea();
        iterations_so_far_jtextarea = new javax.swing.JTextArea();
        jPanel10 = new javax.swing.JPanel();
        jLabel8 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jPanel5 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        number_of_citizens = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        number_of_selected_citizens = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        number_of_bits_per_parameter = new javax.swing.JTextField();
        jLabel4 = new javax.swing.JLabel();
        number_of_parameters = new javax.swing.JTextField();
        jLabel5 = new javax.swing.JLabel();
        number_of_bits_for_muthagen_control = new javax.swing.JTextField();
        jLabel6 = new javax.swing.JLabel();
        number_of_max_iterations_not_useful = new javax.swing.JTextField();
        jPanel6 = new javax.swing.JPanel();
        parameter_values_and_ranges = new javax.swing.JTextArea();
        jPanel4 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        dataset_jTextArea = new javax.swing.JTextArea();
        jPanel11 = new javax.swing.JPanel();
        jLabel7 = new javax.swing.JLabel();
        maximum_acceptable_error_jtextfield = new javax.swing.JTextField();
        jLabel11 = new javax.swing.JLabel();
        fitting_function = new javax.swing.JTextField();

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });
        getContentPane().setLayout(new java.awt.BorderLayout());

        jTabbedPane1.setBackground(new java.awt.Color(153, 255, 204));
        jTabbedPane1.setMinimumSize(new java.awt.Dimension(640, 480));
        jTabbedPane1.setPreferredSize(new java.awt.Dimension(800, 600));

        jPanel1.setLayout(new java.awt.BorderLayout());

        jPanel7.setBackground(new java.awt.Color(255, 255, 102));
        jPanel7.setBorder(javax.swing.BorderFactory.createTitledBorder("Control pane:"));
        jPanel7.setLayout(new java.awt.BorderLayout());

        jButton1.setBackground(new java.awt.Color(0, 204, 204));
        jButton1.setText("GO !");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jPanel7.add(jButton1, java.awt.BorderLayout.CENTER);

        jButton2.setBackground(new java.awt.Color(204, 51, 0));
        jButton2.setText("Stop");
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jPanel7.add(jButton2, java.awt.BorderLayout.EAST);

        jPanel1.add(jPanel7, java.awt.BorderLayout.NORTH);

        grd_display.setBackground(new java.awt.Color(0, 0, 0));
        grd_display.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "XY plot:", 0, 0, null, new java.awt.Color(255, 255, 255)));
        grd_display.setForeground(new java.awt.Color(255, 255, 255));
        grd_display.setLayout(new java.awt.BorderLayout());
        jPanel1.add(grd_display, java.awt.BorderLayout.CENTER);

        jPanel8.setBackground(new java.awt.Color(153, 255, 204));
        jPanel8.setBorder(javax.swing.BorderFactory.createTitledBorder("Info on actual status:"));
        jPanel8.setLayout(new java.awt.BorderLayout());

        jPanel9.setLayout(new java.awt.GridLayout(0, 1));

        best_parameters_value_jtextarea.setBackground(new java.awt.Color(204, 255, 0));
        best_parameters_value_jtextarea.setEditable(false);
        best_parameters_value_jtextarea.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanel9.add(best_parameters_value_jtextarea);

        error_so_far_jtextarea.setBackground(new java.awt.Color(255, 204, 51));
        error_so_far_jtextarea.setEditable(false);
        error_so_far_jtextarea.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanel9.add(error_so_far_jtextarea);

        iterations_so_far_jtextarea.setBackground(new java.awt.Color(255, 124, 0));
        iterations_so_far_jtextarea.setEditable(false);
        iterations_so_far_jtextarea.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanel9.add(iterations_so_far_jtextarea);

        jPanel8.add(jPanel9, java.awt.BorderLayout.CENTER);

        jPanel10.setLayout(new java.awt.GridLayout(0, 1));

        jLabel8.setBackground(new java.awt.Color(204, 255, 0));
        jLabel8.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel8.setText("Best parameter: ");
        jLabel8.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jLabel8.setOpaque(true);
        jPanel10.add(jLabel8);

        jLabel9.setBackground(new java.awt.Color(255, 204, 51));
        jLabel9.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel9.setText("Error so far: ");
        jLabel9.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jLabel9.setOpaque(true);
        jPanel10.add(jLabel9);

        jLabel10.setBackground(new java.awt.Color(255, 124, 0));
        jLabel10.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel10.setText("Additional info: ");
        jLabel10.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jLabel10.setOpaque(true);
        jPanel10.add(jLabel10);

        jPanel8.add(jPanel10, java.awt.BorderLayout.WEST);

        jPanel1.add(jPanel8, java.awt.BorderLayout.SOUTH);

        jTabbedPane1.addTab("App control & Real Time Display", jPanel1);

        jPanel2.setLayout(new java.awt.BorderLayout());

        jPanel5.setLayout(new java.awt.BorderLayout());

        jPanel3.setBackground(new java.awt.Color(255, 204, 0));
        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder("Population properties"));
        jPanel3.setLayout(new java.awt.GridLayout(0, 2));

        jLabel1.setText("Number of citizens:");
        jLabel1.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanel3.add(jLabel1);

        number_of_citizens.setText("300");
        jPanel3.add(number_of_citizens);

        jLabel2.setText("Number of selected citizens:");
        jLabel2.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanel3.add(jLabel2);

        number_of_selected_citizens.setText("11");
        jPanel3.add(number_of_selected_citizens);

        jLabel3.setText("Number of bits per parameter:");
        jLabel3.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanel3.add(jLabel3);

        number_of_bits_per_parameter.setText("32");
        jPanel3.add(number_of_bits_per_parameter);

        jLabel4.setText("Number of parameters:");
        jLabel4.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanel3.add(jLabel4);

        number_of_parameters.setText("3");
        jPanel3.add(number_of_parameters);

        jLabel5.setText("Number of bits for muthagen control:");
        jLabel5.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanel3.add(jLabel5);

        number_of_bits_for_muthagen_control.setText("100");
        jPanel3.add(number_of_bits_for_muthagen_control);

        jLabel6.setText("Narrow the ranges after unuseful iterations:");
        jLabel6.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanel3.add(jLabel6);

        number_of_max_iterations_not_useful.setText("1000000000");
        jPanel3.add(number_of_max_iterations_not_useful);

        jPanel5.add(jPanel3, java.awt.BorderLayout.NORTH);

        jPanel6.setBackground(new java.awt.Color(249, 124, 62));
        jPanel6.setBorder(javax.swing.BorderFactory.createTitledBorder("Initial parameter values and their ranges: "));
        jPanel6.setLayout(new java.awt.BorderLayout());

        parameter_values_and_ranges.setText("0 0 10\n0 0 10\n0 0 1");
        parameter_values_and_ranges.setToolTipText("Each row contains the initial value for parameter followed by its minimum and maximum\nvalues, space separated.");
        parameter_values_and_ranges.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jPanel6.add(parameter_values_and_ranges, java.awt.BorderLayout.CENTER);

        jPanel5.add(jPanel6, java.awt.BorderLayout.CENTER);

        jPanel2.add(jPanel5, java.awt.BorderLayout.CENTER);

        jTabbedPane1.addTab("Population & Parameters", jPanel2);

        jPanel4.setBackground(new java.awt.Color(51, 204, 0));
        jPanel4.setBorder(javax.swing.BorderFactory.createTitledBorder("Dataset values (space separated pairs representing XY values): "));
        jPanel4.setLayout(new java.awt.BorderLayout());

        dataset_jTextArea.setText("0.5 0.822\n1.5 0.617\n3 0.506\n5 0.406\n8 0.365");
        jScrollPane1.setViewportView(dataset_jTextArea);

        jPanel4.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        jPanel11.setBackground(new java.awt.Color(255, 255, 102));
        jPanel11.setBorder(javax.swing.BorderFactory.createTitledBorder("Fitting function properties:"));
        jPanel11.setLayout(new java.awt.GridLayout(0, 2));

        jLabel7.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel7.setText("Maximum acceptable error: ");
        jLabel7.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jLabel7.setHorizontalTextPosition(javax.swing.SwingConstants.LEADING);
        jPanel11.add(jLabel7);

        maximum_acceptable_error_jtextfield.setText("1e-10");
        maximum_acceptable_error_jtextfield.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanel11.add(maximum_acceptable_error_jtextfield);

        jLabel11.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel11.setText("Fitting fucntion:");
        jLabel11.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jLabel11.setHorizontalTextPosition(javax.swing.SwingConstants.LEADING);
        jPanel11.add(jLabel11);
        jLabel11.getAccessibleContext().setAccessibleName("Fitting function");

        fitting_function.setText("a0 * e ^ (-a1 * x) + a2");
        fitting_function.setToolTipText("<html>\nPut here the function to optimize.<br>\nEach parameter must be given in the form of a0, a1, a2, ....<br>\nIf your function is very complex, please consider\ninserting it hardcoded inside the source, as it will\nbe 10 times faster.\n</html>");
        fitting_function.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        fitting_function.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fitting_functionActionPerformed(evt);
            }
        });
        jPanel11.add(fitting_function);

        jPanel4.add(jPanel11, java.awt.BorderLayout.SOUTH);

        jTabbedPane1.addTab("Dataset & Fitting function", jPanel4);

        getContentPane().add(jTabbedPane1, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void fitting_functionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fitting_functionActionPerformed
        // TODO add your handling code here:
        ff.function = fitting_function.getText();
    }//GEN-LAST:event_fitting_functionActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        // TODO add your handling code here:
        g.stop_optimizing = true;
    }//GEN-LAST:event_jButton2ActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        // TODO add your handling code here:
        go();
    }//GEN-LAST:event_jButton1ActionPerformed
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        System.exit(0);
    }//GEN-LAST:event_exitForm
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new ACSE().show();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    public javax.swing.JTextArea best_parameters_value_jtextarea;
    public javax.swing.JTextArea dataset_jTextArea;
    public javax.swing.JTextArea error_so_far_jtextarea;
    public javax.swing.JTextField fitting_function;
    private javax.swing.JPanel grd_display;
    public javax.swing.JTextArea iterations_so_far_jtextarea;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel10;
    private javax.swing.JPanel jPanel11;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JPanel jPanel9;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTabbedPane jTabbedPane1;
    public javax.swing.JTextField maximum_acceptable_error_jtextfield;
    public javax.swing.JTextField number_of_bits_for_muthagen_control;
    public javax.swing.JTextField number_of_bits_per_parameter;
    public javax.swing.JTextField number_of_citizens;
    public javax.swing.JTextField number_of_max_iterations_not_useful;
    public javax.swing.JTextField number_of_parameters;
    public javax.swing.JTextField number_of_selected_citizens;
    public javax.swing.JTextArea parameter_values_and_ranges;
    // End of variables declaration//GEN-END:variables
    
}
