/*
 * @author  Giuseppe Stanghellini @ ismar dot cnr dot it
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

import java.util.Random;
import javax.swing.JTextArea;
import java.util.Calendar;
import java.text.DecimalFormat;
import org.geo.misc.XYGraphPanel;


/**
 *
 * @author  gstanghe
 */
public class Genetic implements Runnable {
    
    Thread my_thread;
    Random my_rnd;
    int NO_INDIVIDUI, NO_GENITORI_BUONI, NO_BITS_PER_MUTAZIONE, MAX_ITER_INUTILI;
    double PARM_RIDUZ;
    Citizen [] citizens;
    ParameterSetup ps;
    boolean stop_optimizing;
    DataSet ds;
    FittingFunction ff;
    XYGraphPanel xygp;
    JTextArea best_parameter_values, iterations_so_far, error_so_far;
    long generated_citizens;
    Calendar cal;
    DecimalFormat df;
    double max_err = 1e-10;
    
    /** Creates a new instance of Genetic */
    public Genetic(int _NO_INDIVIDUI, int _NO_GENITORI_BUONI, int _NO_BITS_PER_MUTAZIONE, int _MAX_ITER_INUTILI, ParameterSetup _ps) 
    {
        generated_citizens = 0;
        best_parameter_values = null; 
        iterations_so_far = null; 
        error_so_far = null;
        my_rnd = new Random();
        ps = _ps;
        NO_INDIVIDUI = _NO_INDIVIDUI;
        NO_GENITORI_BUONI = _NO_GENITORI_BUONI;
        NO_BITS_PER_MUTAZIONE = _NO_BITS_PER_MUTAZIONE;
        MAX_ITER_INUTILI = _MAX_ITER_INUTILI;
        PARM_RIDUZ = 0.2;
        citizens = new Citizen[NO_INDIVIDUI];
        for(int i = 0; i < NO_INDIVIDUI; i++) citizens[i] = new Citizen(ps, i);
        df = new DecimalFormat("000000.00");
    }
    
/**********************************************************************
 *                 GENERA TUTTI I FIGLIOLI
 **********************************************************************/
    void newGeneration()
    {
        int i, j, k;
        long l, num_tot_bits;
        int i_padre, i_madre, i_figlio1, i_figlio2;
   
        /**************************************************************
        * *  I FIGLI SI GENERANO COSI' 
        * * 
        * *           GENITORE A
        * *                        0101111001010110110101001001001
        * *  
        * *           GENITORE B
        * *                        1111111110000111111010101010011
        * * 
        * *                                   ^
        * *       QUI' SI DIVIDE -------------| (bit scelto casualmente)
        * * 
        * * 
        * *           FIGLIO 1     0101111001000111111010101010011
        * * 
        * *           FIGLIO 2     1111111110010110110101001001001
        * * 
        * * 
        * * 
        * * POI UNA CERTA PARTE DELLA POPOLAZIONE LA SI SOTTOPONE
        * * AD AGENTI MUTAGENI.
        * * 
        * ***************************************************************/
   
        /************ I FIGLIOLI SI METTONO IN FONDO ***************/
        k = NO_INDIVIDUI - 1;
   

        /************ GENERA I FIGLIOLI *************/
   	for(i = 0; i < NO_GENITORI_BUONI - 1; i++)
        {
            i_padre = i;
            for(j = i + 1; j < NO_GENITORI_BUONI; j++)
            {
                i_figlio1 = k;
                i_figlio2 = k - 1;
                i_madre = j;
                l = (int)rnd(0, ps.num_params * ps.num_bits);
                
		citizens[i_figlio1] = citizens[i_padre].generate(citizens[i_madre], l);
                citizens[i_figlio1].setId(i_figlio1);
                generated_citizens++;
		citizens[i_figlio2] = citizens[i_madre].generate(citizens[i_padre], l);
                citizens[i_figlio2].setId(i_figlio2);
                generated_citizens++;
	     
                /******** INDICE IN CUI METTERE IL PROSSIMO FIGLIOLO ******/
                k -= 2;
            }
        }

        /*******************************************************************
        * *       FATTI I FIGLI SI CONSERVANO I GENITORI E 
        * *       SI GENERANO CASUALMENTE NUOVI INDIVIDUI. 
        * *******************************************************************/
        iniParamPOP(NO_GENITORI_BUONI + 1, k);
   
        /***********************************************************
        * * BENE ORA BISOGNA APPLICARE
        * * UNA EVENTUALE MUTAZIONE CASUALE
        * **********************************************************
        */
        num_tot_bits = (NO_INDIVIDUI - NO_GENITORI_BUONI) * ps.num_params * ps.num_bits / NO_BITS_PER_MUTAZIONE;
   
        for(i = 0; i < num_tot_bits; i++)
        {
            int individuo_mutare, param_mutare, bit_mutare;
	
            individuo_mutare = (int)rnd(0, NO_INDIVIDUI - NO_GENITORI_BUONI - 1) + NO_GENITORI_BUONI;
            
            param_mutare = (int)rnd(0, ps.num_params);
            bit_mutare = (int)rnd(0, ps.num_bits);
            citizens[individuo_mutare].params.applyMutation(param_mutare, bit_mutare);
        }
   
        /******************************************************************
        * *       INFINE SI TRASFORMANO LE STRINGHE DI BIT NELLE
        * *       RISPETTIVE RAPPRESENTAZIONI DOUBLE
        * ******************************************************************/

        for(i = 0; i < NO_INDIVIDUI; i++)
        {
            citizens[i].params.fromBitsToDouble();
        }        
    }

    double rnd(double x, double y)
    {
        return my_rnd.nextDouble() * (y - x) + x;
    }


    /************************************************************************
    *                INIZIALIZZA IN MODO CASUALE I PARAMETRI
    *                DI PARTENZA PER TUTTA LA POPOLAZIONE
    ************************************************************************/
    void iniParamPOP(int primo_individuo, int ultimo_individuo)
    {
        int i;
        
        for(i = primo_individuo; i < ultimo_individuo; i++)
        {
            citizens[i].params.initWithRandomValue();
            generated_citizens++;
        }
    }

    /************************************************************************
    *            ORDINA LA POPOLAZIONE DAL MIGLIORE AL PEGGIORE
    *            (IN REALTA' PER FARE PRIMA CERCO SOLO I PRIMI
    *             NO_GENITORI_BUONI)
    ************************************************************************/
    void sortOnFITNESS()
    {
        int i, j, k, l;
        Citizen tmp_d;
        for(i = 0; i < NO_GENITORI_BUONI; i++)
        {
            for(j = i + 1; j < NO_INDIVIDUI; j++)
            {
                if(citizens[i].valoreFITNESS > citizens[j].valoreFITNESS)
                {
                    tmp_d = (Citizen)citizens[j];
                    citizens[j] = (Citizen)citizens[i];
                    citizens[i] = (Citizen)tmp_d;
                    citizens[i].setId(i);
                    citizens[j].setId(j);
                }
            }
        }
    }
    
    public void setRunningData(DataSet _ds, FittingFunction _ff, double _max_err, XYGraphPanel _xygp, JTextArea _best_parameter_values, JTextArea _error_so_far, JTextArea _iterations_so_far)
    {
        ds = _ds;
        ff = _ff;
        max_err = _max_err;
        xygp = _xygp;
        best_parameter_values = _best_parameter_values;
        iterations_so_far = _iterations_so_far;
        error_so_far = _error_so_far;
    }

    public void optimize()
    {
        cal = Calendar.getInstance();
        generated_citizens = 0;
        my_thread = new Thread(this, "Optimizing");
        my_thread.start();
    }
    
    public void run()
    {
   	int i, j, iterations, no_cicli_senza_miglioramento;
        double min_err, perc_ampiezze;
        
        stop_optimizing = false;
        iterations = 0;
        
        min_err = -1;
        no_cicli_senza_miglioramento = 0;
        perc_ampiezze = 100;
   
        /********* INIZIALIZZA LA POPOLAZIONE DI PARTENZA IN MANIERA RANDOM ****/
        iniParamPOP(0, NO_INDIVIDUI);
        
        //System.out.println(citizens[0].params.toString());
        //System.exit(0);
        do
        {
            
            /********************************************* GENERA I FIGLIOLI *******/
            newGeneration();
	
            /********** CALCOLA IL FITNESS PER OGNI INDIVIDUO DELLA POPOLAZIONE ****/
            /********** IL FITNESS E' QUEL VALORE PROPORZIONALE ALLA BONTA'     ****/
            /********** DELL'INDIVIDUO IN ESAME.                                ****/
            
            for(i = 0; i < NO_INDIVIDUI; i++)
            {
                ds.calcFITNESS(citizens[i], ff);
            }
	
            /********************* ORDINA I DATI IN FUNZIONE DEL VALORE FITNESS ****/
            sortOnFITNESS();
	
            /*** SE IL MIGLIORE NON CAMBIA DA N CICLI ALLORA ****/
            /*** RESTRINGIAMO SU DI ESSO IL CAMPO DI VARIABILITA' DEI PARAMETRI ****/
            /*** RENDENDOLO IN AMPIEZZA PARI AL 10% DELL'AMPIEZZA PRECEDENTE    ****/
            if(min_err == citizens[0].valoreFITNESS)
            {
                no_cicli_senza_miglioramento++;
                if(no_cicli_senza_miglioramento > MAX_ITER_INUTILI)
                {
                    System.out.println("\n   -----> RESTRINGIMENTO CAMPO DI VARIABILITA ' \n");
                    perc_ampiezze *= PARM_RIDUZ;
                    for(i = 0; i < ps.num_params; i++)
                    {
                        ps.param_min[i] = citizens[0].params.param[i].double_value - 
                             PARM_RIDUZ * (ps.param_max[i] - ps.param_min[i]);
                        ps.param_max[i] = citizens[0].params.param[i].double_value + 
                             PARM_RIDUZ * (ps.param_max[i] - ps.param_min[i]);
                        System.out.println(citizens[0].params.ps.param_min[i] + ", " + citizens[0].params.ps.param_max[i]);
                    }
                    no_cicli_senza_miglioramento = 0;
                    min_err = 1e100;
                }
            }
            else
            {
                /*************** NEL CASO SIA CAMBIATO IL MIGLIORE, 
                * *               FACCIO IL GRAFICO */
                min_err = citizens[0].valoreFITNESS;
                no_cicli_senza_miglioramento = 0;
                ds.display(xygp, 0);
                if(best_parameter_values != null) best_parameter_values.setText(citizens[0].params.toString());
                if(error_so_far != null) error_so_far.setText("" + citizens[0].valoreFITNESS);
            }

            //if(error_so_far != null && no_cicli_senza_miglioramento % 100 == 0) error_so_far.setText("" + citizens[0].valoreFITNESS + " not changing from " + no_cicli_senza_miglioramento + " iterations");
            iterations++;
            
            if(iterations % 100 == 0 && iterations_so_far != null)
            {
                double seconds = Calendar.getInstance().getTimeInMillis() - cal.getTimeInMillis();
                iterations_so_far.setText("Iterations=" + df.format((double)iterations/1000) + "K ; Generated citizens=" + df.format((double)generated_citizens / 1000) + "K" + " ; Iterations / s: " + Math.floor((double)iterations / seconds * 1000 * 100) / 100);
            }
        } 
        while(!stop_optimizing && max_err < min_err);
    }

}
