/*
 * @author  Giuseppe Stanghellini @ ismar dot cnr dot it
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

import javax.swing.*;
import java.math.BigInteger;
import java.math.BigDecimal;
import java.util.Random;

/**
 *
 * @author  gstanghe
 */
public class Parameter implements Cloneable {
    
    public long [] bits;
    public double double_value;
    public long long_value;
    private int id_parameter;
    private ParameterSetup ps;
    
    /** Creates a new instance of Parametri */
    public Parameter(ParameterSetup _ps, int _id_parameter) 
    {
        ps = _ps;
        id_parameter = _id_parameter;
        bits = new long[ps.num_bytes];
        initWithRandomValue();
    }

    public Object clone()
    {
        Parameter _p;
        try
        {
            _p = (Parameter)super.clone();
            _p.bits = (long [])bits.clone();
        }
	catch(Exception e){ _p = null; }
	return _p;
    }
    
    public void initWithRandomValue()
    {
        for(int i = 0; i < ps.num_bytes; i++) bits[i] = (int)(ps.random.nextDouble() * 255);

        fromBitsToDouble();
    }

    public void fromBitsToDouble()
    {
        long rotation = 8, bit;
        long_value = (long)bits[0];
        for(int i = 1; i < ps.num_bytes; i++)
        {
            bit = bits[i] << rotation;
            long_value += bit;
            rotation += 8;
        }
        double_value = ((double)long_value / (double)ps.long_maxval * (ps.param_max[id_parameter] - ps.param_min[id_parameter]) + ps.param_min[id_parameter]);
    }
    
    public void fromDoubleToBits()
    {
        // TODO
    }
}
