/*
 * @author  Giuseppe Stanghellini @ ismar dot cnr dot it
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

import javax.swing.*;
import java.math.BigInteger;
import java.math.BigDecimal;
import java.util.Random;

/**
 *
 * @author  gstanghe
 */
public class ParameterSetup {

    public int num_params, num_bits, num_bytes;
    public double [] param_initial_value, param_min, param_max;
    public Random random;
    public long long_maxval;
    
    
    /** Creates a new instance of ParameterSetup */
    public ParameterSetup(int _num_params, int _num_bits) 
    {
        random = new Random();
        num_params = _num_params;
        num_bits = _num_bits & 0xFFFFFFF8;
        num_bytes = num_bits >> 3;
        long_maxval = (1L << (num_bits - 1)) - 1L;
        param_initial_value = new double[num_params];
        param_min = new double[num_params];
        param_max = new double[num_params];
    }
    
    public void initializeParameters(JTextArea jta)
    {
        String t = jta.getText();
        String [] row = t.split("\n");
        for(int i = 0; i < num_params; i++)
        {
            String [] vr = row[i].split(" ");
            if(java.lang.reflect.Array.getLength(vr) == 3)
            {
                param_initial_value[i] = Double.valueOf(vr[0]).doubleValue();
                param_min[i] = Double.valueOf(vr[1]).doubleValue();
                param_max[i] = Double.valueOf(vr[2]).doubleValue();
            }
        }
    }
    
}
