/*
 * @author  Giuseppe Stanghellini @ ismar dot cnr dot it
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */


/**
 *
 * @author  gstanghe
 */
public class Parameters implements Cloneable
{
    Parameter [] param;
    ParameterSetup ps;
    int id_parameters;
    
    /** Creates a new instance of Parameters */
    public Parameters(ParameterSetup _ps, int _id_parameters)
    {
        id_parameters = _id_parameters;
        ps = _ps;
        param = new Parameter[ps.num_params];
        for(int i = 0; i < ps.num_params; i++) param[i] = new Parameter(ps, i);
    }
    
    public void setId(int _id)
    {
        id_parameters = _id;
    }
    
    public Object clone()
    {
        Parameters _p;
        try
        {
            _p = (Parameters)super.clone();
            _p.param = (Parameter [])param.clone();
            for(int i = 0; i < ps.num_params; i++)
                _p.param[i] = (Parameter)param[i].clone();
        }
	catch(Exception e){ _p = null; }
	return _p;
    }

    public String toString()
    {
        String p = "";
        for(int i = 0; i < ps.num_params; i++)
        {
            if(i == 0)
                p += param[i].double_value;
            else
                p += ", " + param[i].double_value; 
        }
        
        return p;
    }
    
    public void initWithRandomValue()
    {
        for(int j = 0; j < ps.num_params; j++)
        {
            param[j].initWithRandomValue();
        }
    }
    
    public void fromBitsToDouble()
    {
        for(int j = 0; j < ps.num_params; j++) param[j].fromBitsToDouble();
    }
    
    public void applyMutation(int param_mutare, int bit_pos)
    {
        int byte_mutare = bit_pos >> 3;
        int bit_mutare = bit_pos % 8;
        int val_mutare = 1;
        val_mutare <<= bit_mutare;
        param[param_mutare].bits[byte_mutare] ^= val_mutare; 
    }
}
